
#
# Shiny App demonstrating the level calculation of rPref
# ******************************************************
#
# Server File
#
# Press "Run App" in RStudio to start the App
#


library(shiny)
library(rPref)
library(ggplot2)

shinyServer(function(input, output, session) {
  
  # generate the plot
  output$skylinePlot <- renderPlot({
    
  # initalizations
  pref <- high(hp) * high(mpg)
  df <- psel(mtcars, pref, top=nrow(mtcars))
  res <- psel(df, pref)
  m <- aes(x = mpg, y = hp)
   
  # base plot
  p <- ggplot() + geom_point(m, df, cex=4, shape = 21) 
  
  # plot Skyline of current level
  plot_level <- function(p, l) {
    m <- aes(x = mpg, y = hp)
    # Set color value -  must be materialized here
    m$colour <- paste0("Level ", l)
    
    level_front <- subset(df, .level == l)
    steps <- rbind(level_front[,c('mpg','hp')], 
                   data.frame(mpg=c(max(level_front$mpg),min(df$mpg)),
                              hp=c(min(df$hp),max(level_front$hp))))
    p <- p + geom_point(m, level_front, lwd=4) + 
           geom_step(m, steps, direction="vh")
  }
  
  # Plot leved-based Pareto fronts
  if (input$display_frontiers) {
    # plot levels up to current level
    for (l in 1:input$level) { 
     p <- plot_level(p, l)
    }
  } else {
    # plot just the current level
    p <- plot_level(p, input$level)
  }
  
  # set colors
  if (input$level > 1 && input$display_frontiers) {
    colvals <- c(vapply(1:(input$level-1), 
       function(x) {y <- 0.3+(input$level-x)/8; rgb(y,y,y)}, ''), "blue")
  } else {
    colvals <- 'blue'
  }
  
  # map colors to legend names
  if(input$display_frontiers) {
    names(colvals) = paste0("Level ", 1:input$level)
  } else {
    names(colvals) = paste0("Level ", input$level)
  }
  
  # add legend
  p <- p + scale_color_manual(values=colvals, name="Levels")
  p + theme(legend.position=c(1, 1), legend.justification=c(1,1))
    
  })
})